@@ -1,15 +1,16 @@
 
 /*
- *  $Id: defLex.c,v 4.21 2007/07/04 20:51:11 bkorb Exp $
+ *  $Id: defLex.c,v 4.29 2008/07/28 04:51:29 bkorb Exp $
  *
- *  Time-stamp:        "2007-07-04 11:16:53 bkorb"
- *  Last Committed:    $Date: 2007/07/04 20:51:11 $
+ *  Time-stamp:        "2008-07-26 10:06:56 bkorb"
+ *  Last Committed:    $Date: 2008/07/28 04:51:29 $
  *
  *  This module scans the template variable declarations and passes
  *  tokens back to the parser.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -55,7 +56,7 @@ te_dp_event aKeywordTkn[] = { KEYWORD_TABLE };
 #define SET_LIT_TKN(t) lastToken = DP_EV_LIT_ ## t; *(pCurCtx->pzScan++) = NUL;
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by :mkfwd */
+/* static forward declarations maintained by mk-fwd */
 static void
 loadScheme( void );
 
@@ -84,7 +85,7 @@ scanAgain:
      *  We branch here after skipping over a comment
      *  or processing a directive (which may change our context).
      */
-    if (isspace( *pCurCtx->pzScan )) {
+    if (IS_WHITESPACE_CHAR( *pCurCtx->pzScan )) {
         char* pz = pCurCtx->pzScan;
         if (*pz == '\n')
             pCurCtx->lineNo++;
@@ -94,7 +95,7 @@ scanAgain:
          *  This ensures that any names found previously
          *  are NUL terminated.
          */
-        while (isspace(*pz)) {
+        while (IS_WHITESPACE_CHAR(*pz)) {
             if (*pz == '\n')
                 pCurCtx->lineNo++;
             pz++;
@@ -163,7 +164,7 @@ scanAgain:
         if (pCurCtx->pzScan[1] != '<')
             goto BrokenToken;
 
-        pz = assembleHereString( pCurCtx->pzScan + 2 );
+        pz = assembleHereString(pCurCtx->pzScan + 2);
         if (pz == NULL) {
             lastToken = DP_EV_INVALID;
             return DP_EV_INVALID;
@@ -191,7 +192,7 @@ scanAgain:
                     pz = pCurCtx->pzScan + strlen( pCurCtx->pzScan );
                     break;
                 }
-                if (isspace( pz[1] )) {
+                if (IS_WHITESPACE_CHAR( pz[1] )) {
                     *pz = NUL;
                     pz[1] = ';';
                     break;
@@ -206,7 +207,8 @@ scanAgain:
 
     case '`':
     {
-        char* pz = ao_string_cook( pCurCtx->pzScan, &(pCurCtx->lineNo));
+        int line_no = pCurCtx->lineNo;
+        char* pz = ao_string_cook( pCurCtx->pzScan, &line_no);
 
         if (pz == NULL)
             goto NUL_error;
@@ -215,11 +217,10 @@ scanAgain:
 
         pCurCtx->pzScan = pz;
 
-        if (pzShellProgram == NULL)
-            pzShellProgram = getDefine( zShellEnv, AG_TRUE );
-
         lastToken = DP_EV_STRING;
         pz = runShell( (char const*)pz_token );
+        pCurCtx->lineNo = line_no;
+
         if (pz == NULL)
             goto scanAgain;
         TAGMEM( pz, "shell definition string" );
@@ -335,13 +336,21 @@ loadScheme( void )
     char*    pzEnd     = (char*)skipScheme( pzText, pzText + strlen( pzText ));
     char     endCh     = *pzEnd;
     int      schemeLen = (pzEnd - pzText);
+    int      next_ln;
     SCM      res;
 
     /*
      *  NUL terminate the Scheme expression, run it, then restore
      *  the NUL-ed character.
      */
-    *pzEnd = NUL;
+    if (*pzEnd == NUL)
+        AG_ABEND(aprf(zErrMsg, pzProg,
+                      "end of Guile/scheme expression not found",
+                      pCurCtx->pzCtxFname, pCurCtx->lineNo));
+
+    *pzEnd  = NUL;
+    next_ln = pCurCtx->lineNo + count_nl(pzText);
+
     procState = PROC_STATE_GUILE_PRELOAD;
     res = ag_scm_c_eval_string_from_file_line(
         pzText, pCurCtx->pzCtxFname, pCurCtx->lineNo );
@@ -350,6 +359,8 @@ loadScheme( void )
 
     pCurCtx->pzScan = pzEnd;
     pzEnd = (char*)resolveSCM( res ); /* ignore const-ness */
+    pCurCtx->lineNo = next_ln;
+
     if (strlen( pzEnd ) >= schemeLen) {
         AGDUPSTR( pzEnd, pzEnd, "SCM Result" );
 
@@ -396,8 +407,7 @@ alist_to_autogen_def( void )
     }
 
     /*
-     *  NUL terminate the Scheme expression, run it, then restore
-     *  the NUL-ed character.
+     *  Run the scheme expression.  The result is autogen definition text.
      */
     procState = PROC_STATE_GUILE_PRELOAD;
     res = ag_scm_c_eval_string_from_file_line(
@@ -414,7 +424,7 @@ alist_to_autogen_def( void )
     res_len   = AG_SCM_STRLEN( res );
     procState = PROC_STATE_LOAD_DEFS;
     pCurCtx->pzScan = pzEnd;
-    AGFREE( (void*)pzText );
+    AGFREE(pzText);
 
     /*
      *  Now, push the resulting string onto the input stack
@@ -455,9 +465,9 @@ assembleName( char* pzScan, te_dp_event* pRetVal )
      *  Check for a number.
      *  Scan it in and advance "pzScan".
      */
-    if (  isdigit( *pzScan )
+    if (  IS_DEC_DIGIT_CHAR( *pzScan )
        || (  (*pzScan == '-')
-          && isdigit( pzScan[1] )
+          && IS_DEC_DIGIT_CHAR( pzScan[1] )
        )  )  {
         pz_token = pzScan;
         (void)strtol( pzScan, &pzScan, 0 );
@@ -465,67 +475,27 @@ assembleName( char* pzScan, te_dp_event* pRetVal )
         return pzScan;
     }
 
+    if (! IS_UNQUOTABLE_CHAR(*pzScan))
+        AG_ABEND( aprf("%s Error: Invalid input char '%c' in %s on line %d\n",
+                       pzProg, *pzScan, pCurCtx->pzCtxFname, pCurCtx->lineNo));
+
     {
-        static unsigned char zNameChars[ 256 ];
         unsigned char* pz = (unsigned char*)pzScan;
 
-        if (zNameChars[ (unsigned)'a' ] == 0) {
-            /*
-             *  Default to accepting as "OTHER_NAME" all characters
-             */
-            u_int  idx = ((unsigned)' ') + 1;
-            do  {
-                zNameChars[ idx ] = ISNAMECHAR(idx) ? 1 : 2;
-            } while (++idx <= (unsigned)'~');
+        while (IS_VALUE_NAME_CHAR(*pz))          pz++;
 
-            /*
-             *  Now disallow entirely characters we use specially
-             */
-            zNameChars[ (unsigned)'"' ] = 0;
-            zNameChars[ (unsigned)'#' ] = 0;
-            zNameChars[ (unsigned)'(' ] = 0;
-            zNameChars[ (unsigned)')' ] = 0;
-            zNameChars[ (unsigned)',' ] = 0;
-            zNameChars[ (unsigned)';' ] = 0;
-            zNameChars[ (unsigned)'<' ] = 0;
-            zNameChars[ (unsigned)'=' ] = 0;
-            zNameChars[ (unsigned)'>' ] = 0;
-            zNameChars[ (unsigned)'[' ] = 0;
-            zNameChars[ (unsigned)'\''] = 0;
-            zNameChars[ (unsigned)']' ] = 0;
-            zNameChars[ (unsigned)'`' ] = 0;
-            zNameChars[ (unsigned)'{' ] = 0;
-            zNameChars[ (unsigned)'}' ] = 0;
-        }
-
-        /*
-         *  Skip over VAR_NAME characters
-         */
-        while (zNameChars[ *pz ] == 1) pz++;
-
-        /*
-         *  IF the next character terminates the token,
-         *  THEN see if we got any characters at all
-         *  ELSE skip over the rest of the OTHER_NAME
-         */
-        if (zNameChars[ *pz ] == 0) {
-            if (pz == (unsigned char*)pzScan)
-                AG_ABEND( aprf( "%s Error: Invalid input char '%c' "
-                                "in %s on line %d\n", pzProg, *pzScan,
-                                pCurCtx->pzCtxFname, pCurCtx->lineNo ));
-
-            *pRetVal = DP_EV_VAR_NAME;
-        } else {
+        if (IS_UNQUOTABLE_CHAR(*pz)) {
             *pRetVal = DP_EV_OTHER_NAME;
-            while (zNameChars[ *pz ] != 0) pz++;
-        }
+            while (IS_UNQUOTABLE_CHAR(*++pz))    ;
+        } else
+            *pRetVal = DP_EV_VAR_NAME;
 
         /*
          *  Return a NAME token, maybe.
          *  If the name is actually a keyword,
          *  we will return that token code instead.
          */
-        pz_token   = pzScan;
+        pz_token = pzScan;
         pzScan   = (char*)pz;
     }
 
@@ -562,10 +532,12 @@ assembleName( char* pzScan, te_dp_event* pRetVal )
 static char*
 assembleHereString( char* pzScan )
 {
+    static char const endless[] = "Unterminated HereString";
     ag_bool  trimTabs = AG_FALSE;
     char     zMark[ MAX_HEREMARK_LEN ];
     size_t   markLen = 0;
     char*    pzDest;
+    int      here_string_line_no;
 
     /*
      *  See if we are to strip leading tab chars
@@ -578,7 +550,7 @@ assembleHereString( char* pzScan )
     /*
      *  Skip white space up to the marker or EOL
      */
-    while (isspace( *pzScan )) {
+    while (IS_WHITESPACE_CHAR( *pzScan )) {
         if (*pzScan++ == '\n')
             AG_ABEND( aprf( zErrMsg, pzProg, "HereString missing the mark",
                             pCurCtx->pzCtxFname, pCurCtx->lineNo ));
@@ -589,7 +561,7 @@ assembleHereString( char* pzScan )
      */
     {
         char* pz = zMark;
-        while (ISNAMECHAR( *pzScan )) {
+        while (IS_VARIABLE_NAME_CHAR( *pzScan )) {
             if (++markLen >= sizeof(zMark))
                 AG_ABEND( aprf( zErrMsg, pzProg, "HereString mark "
                                 STR( MAX_HEREMARK_LEN ) " or more chars",
@@ -611,13 +583,13 @@ assembleHereString( char* pzScan )
      */
     pzScan = strchr( pzScan, '\n' );
     if (pzScan == NULL)
-        AG_ABEND( aprf( zErrMsg, pzProg, "Unterminated HereString",
-                        pCurCtx->pzCtxFname, pCurCtx->lineNo ));
+        AG_ABEND( aprf( zErrMsg, pzProg, endless, pCurCtx->pzCtxFname,
+                        pCurCtx->lineNo ));
 
     /*
      *  And skip the first new line + conditionally skip tabs
      */
-    pCurCtx->lineNo++;
+    here_string_line_no = pCurCtx->lineNo++;
     pzScan++;
 
     if (trimTabs)
@@ -629,7 +601,7 @@ assembleHereString( char* pzScan )
      *  DO copy characters
      */
     while (  (strncmp( pzScan, zMark, markLen ) != 0)
-          || ISNAMECHAR( pzScan[ markLen ]) )  {
+          || IS_VARIABLE_NAME_CHAR( pzScan[ markLen ]) )  {
 
         for (;;) {
             switch (*(pzDest++) = *(pzScan++)) {
@@ -638,8 +610,8 @@ assembleHereString( char* pzScan )
                 goto lineDone;
 
             case NUL:
-                AG_ABEND( aprf( zErrMsg, pzProg, "Unterminated HereString",
-                                pCurCtx->pzCtxFname, pCurCtx->lineNo ));
+                AG_ABEND( aprf( zErrMsg, pzProg, endless, pCurCtx->pzCtxFname,
+                                here_string_line_no));
             }
         } lineDone:;
 
