@@ -1,7 +1,7 @@
 
 /*
- *  $Id: boolean.c,v 4.12 2007/07/04 21:36:37 bkorb Exp $
- * Time-stamp:      "2007-07-04 11:33:18 bkorb"
+ *  $Id: boolean.c,v 4.15 2008/08/04 01:01:52 bkorb Exp $
+ * Time-stamp:      "2008-08-03 13:06:02 bkorb"
  *
  *   Automated Options Paged Usage module.
  *
@@ -10,7 +10,8 @@
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -47,6 +48,14 @@ optionBooleanVal( tOptions* pOpts, tOptDesc* pOD )
     char* pz;
     ag_bool  res = AG_TRUE;
 
+    if ((pOD->fOptState & OPTST_RESET) != 0)
+        return;
+
+    if (pOD->optArg.argString == NULL) {
+        pOD->optArg.argBool = AG_FALSE;
+        return;
+    }
+
     switch (*(pOD->optArg.argString)) {
     case '0':
     {
