@@ -1,7 +1,7 @@
 
 /*
- *  $Id: load.c,v 4.24 2007/11/13 05:49:47 bkorb Exp $
- *  Time-stamp:      "2007-11-12 20:38:32 bkorb"
+ *  $Id: load.c,v 4.26 2008/06/14 22:24:22 bkorb Exp $
+ *  Time-stamp:      "2008-06-14 11:11:37 bkorb"
  *
  *  This file contains the routines that deal with processing text strings
  *  for options, either from a NUL-terminated string passed in or from an
@@ -9,7 +9,8 @@
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -338,7 +339,7 @@ mungeString( char* pzTxt, tOptionLoadMode mode )
 static char*
 assembleArgValue( char* pzTxt, tOptionLoadMode mode )
 {
-    tSCC zBrk[] = " \t:=";
+    tSCC zBrk[] = " \t\n:=";
     char* pzEnd = strpbrk( pzTxt, zBrk );
     int   space_break;
 
